@@ -1,8 +1,10 @@
 /*
  * %CopyrightBegin%
- * 
- * Copyright Ericsson AB 1997-2021. All Rights Reserved.
- * 
+ *
+ * SPDX-License-Identifier: Apache-2.0
+ *
+ * Copyright Ericsson AB 1997-2025. All Rights Reserved.
+ *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
@@ -14,7 +16,7 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
+ *
  * %CopyrightEnd%
  */
 /*
@@ -48,7 +50,6 @@ static char* win32_errorstr(int error);
 static int has_console(void);
 static char** fnuttify_argv(char **argv);
 static void free_fnuttified(char **v);
-static int windowed = 0;
 
 #ifdef LOAD_BEAM_DYNAMICALLY
 typedef int SysGetKeyFunction(int);
@@ -133,103 +134,6 @@ free_env_val(char *value)
 	free(value);
 }
 
-
-int
-start_win_emulator(char* utf8emu, char *utf8start_prog, char** utf8argv, int start_detached)
-{
-    int len;
-    int argc = 0;
-
-    windowed = 1;
-    while (utf8argv[argc] != NULL) {
-	++argc;
-    }
-
-    if (start_detached) {
-	wchar_t *start_prog=NULL;
-	int result;
-	int i;
-	wchar_t **argv;
-	close(0);
-	close(1);
-	close(2);
-	
-	set_env("ERL_CONSOLE_MODE", "detached");
-	set_env(DLL_ENV, utf8emu);
-
-	utf8argv[0] = utf8start_prog;
-	utf8argv = fnuttify_argv(utf8argv);
-
-	len = MultiByteToWideChar(CP_UTF8, 0, utf8start_prog, -1, NULL, 0);
-	start_prog = malloc(len*sizeof(wchar_t));
-	MultiByteToWideChar(CP_UTF8, 0, utf8start_prog, -1, start_prog, len);
-
-	/* Convert utf8argv to multibyte argv */
-	argv = malloc((argc+1) * sizeof(wchar_t*));
-	for (i=0; i<argc; i++) {
-	    len = MultiByteToWideChar(CP_UTF8, 0, utf8argv[i], -1, NULL, 0);
-	    argv[i] = malloc(len*sizeof(wchar_t));
-	    MultiByteToWideChar(CP_UTF8, 0, utf8argv[i], -1, argv[i], len);
-	}
-	argv[argc] = NULL;
-
-#ifdef ARGS_HARDDEBUG
-	{
-	    wchar_t tempbuf[2048] = L"";
-	    wchar_t *sbuf;
-	    int i;
-	    sbuf=tempbuf;
-	    sbuf += swprintf(sbuf, 2048, L"utf16: %s\n", start_prog);
-	    for (i = 0; i < argc; ++i) {
-		sbuf += swprintf(sbuf, 2048, L"|%s|", argv[i]);
-	    };
-	    sbuf += swprintf(sbuf, 2048, L"\nutf8: \n");
-	    for (i = 0; i < argc; ++i) {
-		sbuf += swprintf(sbuf, 2048, L"|%S|", utf8argv[i]);
-	    };
-	    MessageBoxW(NULL, tempbuf, L"respawn args", MB_OK|MB_ICONERROR);
-	}
-#endif
-
-	result = _wspawnv(_P_DETACH, start_prog, argv);
-	free_fnuttified(utf8argv);
-	if (result == -1) {
-	    error("Failed to execute %S: %s", start_prog, win32_errorstr(_doserrno));
-	}
-    } else {
-	wchar_t *emu=NULL;
-#ifdef LOAD_BEAM_DYNAMICALLY
-	HMODULE beam_module = NULL;
-	len = MultiByteToWideChar(CP_UTF8, 0, utf8emu, -1, NULL, 0);
-	emu = malloc(len*sizeof(wchar_t));
-	MultiByteToWideChar(CP_UTF8, 0, utf8emu, -1, emu, len);
-#ifdef ARGS_HARDDEBUG
-	{
-	    char sbuf[2048] = "";
-	    int i;
-	    strcat(sbuf,utf8emu);
-	    strcat(sbuf,":");
-	    for (i = 0; i < argc; ++i) {
-		strcat(sbuf,"|");
-		strcat(sbuf, utf8argv[i]);
-		strcat(sbuf,"| ");
-	    }
-	    MessageBox(NULL, sbuf, "erl_start args", MB_OK|MB_ICONERROR);
-	}
-#endif
-	beam_module = load_win_beam_dll(emu);
-#endif	
-	set_env("ERL_CONSOLE_MODE", "window");
-#ifdef LOAD_BEAM_DYNAMICALLY
-	(*sys_primitive_init_p)(beam_module);
-	(*erl_start_p)(argc,utf8argv);
-#else
-	erl_start(argc,utf8argv);
-#endif
-    }
-    return 0;
-}
-
 void __cdecl 
 do_keep_window(void)
 {
@@ -244,8 +148,6 @@ do_keep_window(void)
 int
 start_emulator(char* utf8emu, char *utf8start_prog, char** utf8argv, int start_detached)
 {
-    static char console_mode[] = "tty:ccc";
-    char* fd_type;
     char* title;
     int len;
     int argc = 0;
@@ -254,8 +156,6 @@ start_emulator(char* utf8emu, char *utf8start_prog, char** utf8argv, int start_d
     fprintf(stderr,"utf8emu = %s, start_prog = %s\n", utf8emu, utf8start_prog);
 #endif
 
-    fd_type = strchr(console_mode, ':');
-    fd_type++;
     _flushall();
 
     while (utf8argv[argc] != NULL) {
@@ -312,7 +212,14 @@ start_emulator(char* utf8emu, char *utf8start_prog, char** utf8argv, int start_d
 
 	if (result == (HANDLE)-1) {
 #ifdef ARGS_HARDDEBUG
-	    MessageBox(NULL, "_wspawnv failed","Start detached",MB_OK);
+            wchar_t buf[256], buffer[256], path[1024];
+            DWORD err = GetLastError();
+            FormatMessageW(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
+                           NULL, err, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
+                           buf, (sizeof(buf) / sizeof(wchar_t)), NULL);
+            GetEnvironmentVariableW(L"PATH",path,sizeof(path) / sizeof(wchar_t));
+            wsprintfW(buffer,L"_wspawnv failed %s(%d)\nPATH=%s", buf, err, path);
+	    MessageBoxW(NULL, buffer , L"Start detached",MB_OK);
 #endif
 	    return 1;
 	}
@@ -339,7 +246,7 @@ start_emulator(char* utf8emu, char *utf8start_prog, char** utf8argv, int start_d
 	}
 #endif
 	beam_module = load_win_beam_dll(emu);
-#endif	
+#endif
 
 	/*
 	 * Start the emulator.
@@ -350,8 +257,8 @@ start_emulator(char* utf8emu, char *utf8start_prog, char** utf8argv, int start_d
 	    SetConsoleTitle(title);
 	}
 	free_env_val(title);
-	
-	set_env("ERL_CONSOLE_MODE", console_mode);
+
+	set_env("ERL_CONSOLE_MODE", "windowed");
 	if (keep_window) {
 	    atexit(do_keep_window);
 	}
@@ -387,10 +294,8 @@ error(char* format, ...)
     vsprintf(sbuf, format, ap);
     va_end(ap);
 
-    if (!windowed && has_console()) {
+    if (has_console()) {
 	fprintf(stderr, "%s\n", sbuf);
-    } else {
-	MessageBox(NULL, sbuf, "Werl", MB_OK|MB_ICONERROR);
     }
     exit(1);
 }
