@@ -1,7 +1,9 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2010-2022. All Rights Reserved.
+ * SPDX-License-Identifier: Apache-2.0
+ *
+ * Copyright Ericsson AB 2010-2025. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,9 +26,12 @@
 #ifdef HAVE_EC
 # if defined(HAS_3_0_API)
 
+# include <openssl/core_names.h>
+
 int get_curve_definition(ErlNifEnv* env, ERL_NIF_TERM *ret, ERL_NIF_TERM def,
                          OSSL_PARAM params[], int *i,
-                         size_t *order_size)
+                         size_t *order_size,
+                         struct get_curve_def_ctx* gcd)
 {
     const ERL_NIF_TERM* curve;
     int c_arity = -1;
@@ -38,6 +43,7 @@ int get_curve_definition(ErlNifEnv* env, ERL_NIF_TERM *ret, ERL_NIF_TERM def,
     int arity = -1;
     const ERL_NIF_TERM* curve_tuple;
 
+
     /* Here are two random curve definition examples, one prime_field and
        one characteristic_two_field. Both are from the crypto/src/crypto_ec_curves.erl.
 
@@ -77,6 +83,23 @@ int get_curve_definition(ErlNifEnv* env, ERL_NIF_TERM *ret, ERL_NIF_TERM def,
         c_arity != 5)
         assign_goto(*ret, err, EXCP_ERROR_N(env, 1, "Bad curve def. Expect 5-tuple."));
 
+    if (gcd->use_curve_name
+        && curve_tuple[1] != atom_undefined
+        && enif_get_atom(env, curve_tuple[1], gcd->curve_name,
+                         sizeof(gcd->curve_name), ERL_NIF_LATIN1)) {
+        ErlNifBinary order_bin;
+        params[(*i)++] = OSSL_PARAM_construct_utf8_string("group", gcd->curve_name, 0);
+
+        if (order_size) {
+            if (!enif_inspect_binary(env, curve[3], &order_bin))
+                assign_goto(*ret, err, EXCP_ERROR_N(env, 1, "Bad order"));
+            *order_size = order_bin.size;
+        }
+        gcd->use_curve_name = 1;
+        return 1;
+    }
+    gcd->use_curve_name = 0;
+
     if (!get_ossl_octet_string_param_from_bin(env, "generator", curve[2], &params[(*i)++]))
         assign_goto(*ret, err, EXCP_ERROR_N(env, 1, "Bad Generator (Point)"));
 
@@ -187,15 +210,8 @@ int get_curve_definition(ErlNifEnv* env, ERL_NIF_TERM *ret, ERL_NIF_TERM def,
         } else
             assign_goto(*ret, err, EXCP_ERROR_N(env, 1, "Bad last field"));
 
-        {
-            ErlNifBinary tmp;
-                        
-            if (!enif_inspect_binary(env, bin_from_bn(env,p), &tmp) || // Allocate buf
-                BN_bn2nativepad(p, tmp.data, tmp.size) < 0) {// Fill with BN in right endianity
-                assign_goto(*ret, err, EXCP_ERROR_N(env, 1, "BN padding failed"));
-            }
-            params[(*i)++] = OSSL_PARAM_construct_BN("p", tmp.data, tmp.size);
-        }
+        if (!get_ossl_BN_param_from_bn(env, "p", p, &params[(*i)++]))
+            assign_goto(*ret, err, EXCP_ERROR_N(env, 1, "BN padding failed"));
 #  endif
     }
     else
@@ -216,6 +232,7 @@ int get_ec_public_key(ErlNifEnv* env, ERL_NIF_TERM key, EVP_PKEY **pkey)
     int tpl_arity;
     int i = 0;
     OSSL_PARAM params[15];
+    struct get_curve_def_ctx gcd;
     EVP_PKEY_CTX *pctx = NULL;
     
     if (!enif_get_tuple(env, key, &tpl_arity, &tpl_terms) ||
@@ -227,19 +244,27 @@ int get_ec_public_key(ErlNifEnv* env, ERL_NIF_TERM key, EVP_PKEY **pkey)
     if (!get_ossl_octet_string_param_from_bin(env, "pub",  tpl_terms[1], &params[i++]))
         assign_goto(ret, err, EXCP_BADARG_N(env, 0, "Bad public key"));
 
-    if (!get_curve_definition(env, &ret, tpl_terms[0], params, &i, NULL))
+    if (!(pctx = EVP_PKEY_CTX_new_from_name(NULL, "EC", NULL)))
+        assign_goto(ret, err, EXCP_ERROR(env, "Can't make EVP_PKEY_CTX"));
+
+    gcd.use_curve_name = 1;
+retry_without_name:
+    if (!get_curve_definition(env, &ret, tpl_terms[0], params, &i, NULL, &gcd))
         goto err;
 
     params[i++] = OSSL_PARAM_construct_end();
 
-    if (!(pctx = EVP_PKEY_CTX_new_from_name(NULL, "EC", NULL)))
-        assign_goto(ret, err, EXCP_ERROR(env, "Can't make EVP_PKEY_CTX"));
-
     if (EVP_PKEY_fromdata_init(pctx) <= 0)
         assign_goto(ret, err, EXCP_ERROR(env, "Can't init fromdata"));
     
-    if (EVP_PKEY_fromdata(pctx, pkey, EVP_PKEY_PUBLIC_KEY, params) <= 0)
+    if (EVP_PKEY_fromdata(pctx, pkey, EVP_PKEY_PUBLIC_KEY, params) <= 0) {
+        if (gcd.use_curve_name) {
+            gcd.use_curve_name = 0;
+            i = 1;
+            goto retry_without_name;
+        }
         assign_goto(ret, err, EXCP_ERROR(env, "Can't do fromdata"));
+    }
 
     if (!*pkey)
         assign_goto(ret, err, EXCP_ERROR(env, "Couldn't get a public key"));
@@ -253,13 +278,7 @@ int get_ec_public_key(ErlNifEnv* env, ERL_NIF_TERM key, EVP_PKEY **pkey)
 }
 
 
-int get_ec_private_key_2(ErlNifEnv* env,
-                         ERL_NIF_TERM curve, ERL_NIF_TERM key,
-                         EVP_PKEY **pkey,
-                         ERL_NIF_TERM *ret,
-                         size_t *order_size);
-
-int get_ec_private_key_2(ErlNifEnv* env,
+static int get_ec_private_key_2(ErlNifEnv* env,
                          ERL_NIF_TERM curve, ERL_NIF_TERM key,
                          EVP_PKEY **pkey,
                          ERL_NIF_TERM *ret,
@@ -267,24 +286,33 @@ int get_ec_private_key_2(ErlNifEnv* env,
 {
     int i = 0;
     OSSL_PARAM params[15];
+    struct get_curve_def_ctx gcd;
     EVP_PKEY_CTX *pctx = NULL;
 
     if (!get_ossl_BN_param_from_bin(env, "priv",  key, &params[i++]))
         assign_goto(*ret, err, EXCP_BADARG_N(env, 0, "Bad private key"));
 
-    if (!get_curve_definition(env, ret, curve, params, &i, order_size))
+    if (!(pctx = EVP_PKEY_CTX_new_from_name(NULL, "EC", NULL)))
+        assign_goto(*ret, err, EXCP_ERROR(env, "Can't make EVP_PKEY_CTX"));
+
+    gcd.use_curve_name = 1;
+retry_without_name:
+    if (!get_curve_definition(env, ret, curve, params, &i, order_size, &gcd))
         goto err;
 
     params[i++] = OSSL_PARAM_construct_end();
 
-    if (!(pctx = EVP_PKEY_CTX_new_from_name(NULL, "EC", NULL)))
-        assign_goto(*ret, err, EXCP_ERROR(env, "Can't make EVP_PKEY_CTX"));
-
     if (EVP_PKEY_fromdata_init(pctx) <= 0)
         assign_goto(*ret, err, EXCP_ERROR(env, "Can't init fromdata"));
     
-    if (EVP_PKEY_fromdata(pctx, pkey, EVP_PKEY_KEYPAIR, params) <= 0)
+    if (EVP_PKEY_fromdata(pctx, pkey, EVP_PKEY_KEYPAIR, params) <= 0) {
+        if (gcd.use_curve_name) {
+            gcd.use_curve_name = 0;
+            i = 1;
+            goto retry_without_name;
+        }
         assign_goto(*ret, err, EXCP_ERROR(env, "Can't do fromdata"));
+    }
 
     if (!*pkey)
         assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get a private key"));
@@ -319,16 +347,18 @@ int get_ec_private_key(ErlNifEnv* env, ERL_NIF_TERM key, EVP_PKEY **pkey)
     return 0;
 }
 
-int mk_pub_key_binary(ErlNifEnv* env, EVP_PKEY **peer_pkey, ErlNifBinary *pubkey_bin, ERL_NIF_TERM *ret);
+static int mk_pub_key_binary(ErlNifEnv* env, EVP_PKEY *peer_pkey,
+                             ErlNifBinary *pubkey_bin, ERL_NIF_TERM *ret);
 
 ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
 { /* (Curve, PrivKey|undefined)  */
     ERL_NIF_TERM ret = atom_undefined;
     int i = 0;
     OSSL_PARAM params[15];
+    struct get_curve_def_ctx gcd;
     EVP_PKEY_CTX *pctx = NULL;
     EVP_PKEY *pkey = NULL, *peer_pkey = NULL;
-    size_t sz, order_size;
+    size_t sz, order_size = 0;
     BIGNUM *priv_bn = NULL;
     ErlNifBinary pubkey_bin;
     
@@ -339,9 +369,8 @@ ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM ar
             
             /* Get the two keys, pub as binary and priv as BN.
                Since the private key is explicitly given, it must be calculated.
-               I haven't found any way to do that with the pure 3.0 interface yet.
             */
-            if (!mk_pub_key_binary(env, &peer_pkey, &pubkey_bin, &ret))
+            if (!mk_pub_key_binary(env, peer_pkey, &pubkey_bin, &ret))
                 goto err;
 
             if (!EVP_PKEY_get_bn_param(peer_pkey, "priv", &priv_bn))
@@ -349,26 +378,36 @@ ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM ar
         }
     else
         {
+            /* Neither the private nor the public key is known, so we generate the pair: */
+            if (!(pctx = EVP_PKEY_CTX_new_from_name(NULL, "EC", NULL)))
+                assign_goto(ret, err, EXCP_ERROR(env, "Can't EVP_PKEY_CTX_new_from_name"));
+
+            gcd.use_curve_name = 1;
+    retry_without_name:
             /* PrivKey (that is, argv[1]) == atom_undefined */
-            if (!get_curve_definition(env, &ret, argv[0], params, &i, &order_size))
+            if (!get_curve_definition(env, &ret, argv[0], params, &i,
+                                      &order_size, &gcd))
                 // INSERT "ret" parameter in get_curve_definition !!
                 assign_goto(ret, err, EXCP_BADARG_N(env, 0, "Couldn't get Curve definition"));
     
             params[i++] = OSSL_PARAM_construct_end();
 
-            /* Neither the private nor the public key is known, so we generate the pair: */
-            if (!(pctx = EVP_PKEY_CTX_new_from_name(NULL, "EC", NULL)))
-                assign_goto(ret, err, EXCP_ERROR(env, "Can't EVP_PKEY_CTX_new_from_name"));
-
             if (EVP_PKEY_keygen_init(pctx) <= 0)
                 assign_goto(ret, err, EXCP_ERROR(env, "Can't EVP_PKEY_keygen_init"));
 
             if (!EVP_PKEY_CTX_set_params(pctx, params))
                 assign_goto(ret, err, EXCP_ERROR(env, "Can't EVP_PKEY_CTX_set_params"));
-        
-            if (!EVP_PKEY_generate(pctx, &pkey))
+
+            if (!EVP_PKEY_generate(pctx, &pkey)) {
+                if (gcd.use_curve_name) {
+                    gcd.use_curve_name = 0;
+                    i = 0;
+                    goto retry_without_name;
+                }
                 assign_goto(ret, err, EXCP_ERROR(env, "Couldn't generate EC key"));
-    
+            }
+
+
             /* Get the two keys, pub as binary and priv as BN */
             if (!EVP_PKEY_get_octet_string_param(pkey, "encoded-pub-key", NULL, 0, &sz))
                 assign_goto(ret, err, EXCP_ERROR(env, "Can't get pub octet string size"));
@@ -386,6 +425,8 @@ ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM ar
                 assign_goto(ret, err, EXCP_BADARG_N(env, 1, "Couldn't get priv key bytes"));
         }
 
+    if (order_size == 0)
+        order_size = BN_num_bytes(priv_bn);
     ret = enif_make_tuple2(env,
                            enif_make_binary(env, &pubkey_bin),
                            bn2term(env, order_size, priv_bn));
@@ -398,67 +439,81 @@ ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM ar
     return ret;
 }
 
-int mk_pub_key_binary(ErlNifEnv* env, EVP_PKEY **peer_pkey, ErlNifBinary *pubkey_bin, ERL_NIF_TERM *ret)
+static int mk_pub_key_binary(ErlNifEnv* env, EVP_PKEY *peer_pkey,
+                             ErlNifBinary *pubkey_bin, ERL_NIF_TERM *ret)
 {
-    EC_KEY *ec_key = NULL;
-    EC_POINT *public_key = NULL;
-    EC_GROUP *group = NULL;
-    BIGNUM *priv_bn = NULL;
-    
-    *ret = atom_undefined;
+    size_t pub_key_size = 0;
+    size_t group_name_size = 0;
+    char group_name_buf[20];
+    char* group_name = group_name_buf;
+    int group_nid;
+    EC_GROUP* ec_group = NULL;
+    EC_POINT* pub_key = NULL;
+    BIGNUM* priv_bn = NULL;
+    int ok = 0;
 
-    /* Use the deprecated interface to get the curve and
-       private key in pre 3.0 form: */
-    if ((ec_key = EVP_PKEY_get1_EC_KEY(*peer_pkey)) == NULL)
-        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get EC key"));
+    /* This code was inspired by
+     * https://github.com/openssl/openssl/issues/18437
+     * which first tried to get public key directly with
+     * EVP_PKEY_get_octet_string_param(peer_pkey, OSSL_PKEY_PARAM_PUB_KEY,..)
+     *
+     * I removed that since I don't know what key format that will produce
+     * if it succeeds. That is, we go directly to the "fallback" and calculate
+     * the public key.
+     */
 
-    if ((group = EC_GROUP_dup(EC_KEY_get0_group(ec_key))) == NULL)
+    if (!EVP_PKEY_get_utf8_string_param(peer_pkey, OSSL_PKEY_PARAM_GROUP_NAME,
+                                        NULL, 0, &group_name_size))
+        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get EC group name size"));
+
+    if (group_name_size >= sizeof(group_name_buf))
+        group_name = enif_alloc(group_name_size + 1);
+    if (!EVP_PKEY_get_utf8_string_param(peer_pkey, OSSL_PKEY_PARAM_GROUP_NAME,
+                                            group_name, group_name_size+1,
+                                            NULL))
+        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get EC group name"));
+
+    group_nid = OBJ_sn2nid(group_name);
+    if (group_nid == NID_undef)
+        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get EC group nid"));
+
+    ec_group = EC_GROUP_new_by_curve_name(group_nid);
+    if (ec_group == NULL)
         assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get EC_GROUP"));
 
-    if ((public_key = EC_POINT_new(group)) == NULL)
+    pub_key = EC_POINT_new(ec_group);
+    if (pub_key == NULL)
         assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't create POINT"));
 
-    if (!EC_POINT_copy(public_key, EC_GROUP_get0_generator(group)))
-        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't copy POINT"));
-
-    /* Make the corresponding public key */
-    if (!EVP_PKEY_get_bn_param(*peer_pkey, "priv", &priv_bn))
+    if (!EVP_PKEY_get_bn_param(peer_pkey, OSSL_PKEY_PARAM_PRIV_KEY, &priv_bn))
         assign_goto(*ret, err, EXCP_BADARG_N(env, 1, "Couldn't get peer priv key bytes"));
 
-    if (BN_is_zero(priv_bn))
-        assign_goto(*ret, err, EXCP_BADARG_N(env, 1, "peer priv key must not be 0"));
-
-    if (!EC_POINT_mul(group, public_key, priv_bn, NULL, NULL, NULL))
+    if (!EC_POINT_mul(ec_group, pub_key, priv_bn, NULL, NULL, NULL))
         assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't multiply POINT"));
 
-    if (!EC_KEY_set_public_key(ec_key, public_key))
-        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't set EC_KEY"));
+    pub_key_size = EC_POINT_point2oct(ec_group, pub_key,
+                                      POINT_CONVERSION_UNCOMPRESSED, NULL, 0, NULL);
+    if (pub_key_size == 0)
+        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get pub_key_size"));
 
-    if (!EVP_PKEY_assign_EC_KEY(*peer_pkey, ec_key))
-        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't assign EC_KEY to PKEY"));
-            
-    /* And now get the binary representation (by some reason we can't read it from
-       peer_pubkey in the calling function with 3.0-functions.)
-    */
-    {
-        point_conversion_form_t form = EC_KEY_get_conv_form(ec_key);
-        size_t dlen = EC_POINT_point2oct(group, public_key, form, NULL, 0, NULL);
-
-        if (!enif_alloc_binary(dlen, pubkey_bin) ||
-            !EC_POINT_point2oct(group, public_key, form, pubkey_bin->data, pubkey_bin->size, NULL)
-            )
-            assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get public key"));
+    enif_alloc_binary(pub_key_size, pubkey_bin);
+    if (!EC_POINT_point2oct(ec_group, pub_key, POINT_CONVERSION_UNCOMPRESSED,
+                            pubkey_bin->data,
+                            pubkey_bin->size, NULL)) {
+        enif_release_binary(pubkey_bin);
+        assign_goto(*ret, err, EXCP_ERROR(env, "Couldn't get pub key bytes"));
     }
 
- err:
-    if (public_key) EC_POINT_free(public_key);
-    if (group) EC_GROUP_free(group);
+    *ret = enif_make_binary(env, pubkey_bin);
+    ok = 1;
+
+err:
+    if (group_name != group_name_buf) enif_free(group_name);
+    if (pub_key) EC_POINT_free(pub_key);
+    if (ec_group) EC_GROUP_free(ec_group);
     if (priv_bn) BN_free(priv_bn);
 
-    if (*ret == atom_undefined)
-        return 1;
-    else
-        return 0;
+    return ok;
 }
     
 # endif /* HAS_3_0_API */
@@ -908,10 +963,8 @@ ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM ar
 
 # endif /* ! HAS_3_0_API */
 
-#endif /* HAVE_EC */
+#else /* ifndef HAVE_EC */
 
-
-#if ! defined(HAVE_EC)
 ERL_NIF_TERM ec_generate_key_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
 { /* (Curve, PrivKey)  */
     return EXCP_NOTSUP_N(env, 0, "EC not supported");
