@@ -1,7 +1,9 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2021. All Rights Reserved.
+%% SPDX-License-Identifier: Apache-2.0
+%%
+%% Copyright Ericsson AB 2008-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,6 +22,7 @@
 
 %%
 -module(dbg_wx_mon).
+-moduledoc false.
 
 -include_lib("kernel/include/file.hrl").
 -include_lib("wx/include/wx.hrl").
@@ -355,7 +358,7 @@ gui_cmd('Next', State) ->
     int:next((State#state.focus)#pinfo.pid),
     State;
 gui_cmd('Continue', State) ->
-    int:continue((State#state.focus)#pinfo.pid),
+    _ = int:continue((State#state.focus)#pinfo.pid),
     State;
 gui_cmd('Finish ', State) ->
     int:finish((State#state.focus)#pinfo.pid),
@@ -714,7 +717,7 @@ load_settings2(Settings, State) ->
     lists:foreach(fun(Break) ->
 			  {{Mod, Line}, [Status, Action, _, Cond]} =
 			      Break,
-			  int:break(Mod, Line),
+			  _ = int:break(Mod, Line),
 			  if
 			      Status =:= inactive ->
 				  int:disable_break(Mod, Line);
